package kawa.lang;

/** Encapsulate multiple values in a single object.
 * In Scheme and Lisp mainly used to return multiple values from a function.
 */

public class Values implements Printable
{
  private Object[] vals;

  /** Constructor.
   * @param values the values to encapulate
   */
  public Values (Object[] values)
  {
    vals = values;
  }

  /** Get the values encapsulated. */
  public Object[] values ()
  {
    return vals;
  }

  /** Apply a Procedure with these values as the arguments. */
  public Object call_with (Procedure proc)
       throws WrongArguments, WrongType, GenericError, UnboundSymbol
  {
    return proc.applyN (vals);
  }

  public void print(java.io.PrintStream ps)
  {
    int size = vals.length;
    ps.print("#<values");
    for (int i = 0; i < size; i++)
      {
	ps.print (" ");
	kawa.lang.print.print (vals[i], ps);
      }
    ps.print (">");
  }
}
